'use strict';
angular.module('aom.directives')
  .directive('aomBarcode', function ($ionicActionSheet, $timeout, $cordovaBarcodeScanner, aomResourcesSrv, aomComponentsSrv, aomJobSrv, aomLogSrv, aomBackSrv) {
    return {
      scope: {
        field: '=',
        readonly: '=',
        shared: '=',
        changeToFieldController: '=',
        scrollTo: '='
      },
      link: function ($scope, ele, attrs) {

        function showBarcodeOptions(field) {      
          var options = [
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.EnterManually, key: 'Manually' },
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.Scan, key: 'Scan' }
          ];
          $ionicActionSheet.show({
            buttons: options,
            titleText: aomResourcesSrv.data.resources.AomActionSheetOptions.Title,
            cancelText: aomResourcesSrv.data.resources.Common.Cancel,
            cancel: function () {
              return true;
            },
            buttonClicked: function (index, object) {
              if (object.key === 'Manually') {
                $timeout(function () {
                  if (ionic.Platform.isIOS() && $scope.shared !== undefined) {
                    $scope.shared.keyboardUp = true;
                    $scope.scrollTo(attrs.id);
                  }
                  field.manuallyEnterBarcode = true;
                  aomComponentsSrv.showKeyboard();
                });
              } else if (object.key === 'Scan') {
                scan();
              }
              return true;
            }
          });
        }

        function platformDependentCancelData() {
          return ionic.Platform.isIOS() ? 0 : false;
        }

        function scan() {
          aomBackSrv.init(cancelScan);
          $cordovaBarcodeScanner.scan()
            .then(function (barcodeData) {
              if (barcodeData !== undefined && barcodeData !== null && barcodeData.cancelled === platformDependentCancelData()) {
                aomJobSrv.data.selectedField.Data = barcodeData.text;
                aomJobSrv.calc(aomJobSrv.data.selectedField.Name);
                aomBackSrv.exit(true);
              }
            }, function (err) {
              aomLogSrv.log({ message: 'Error scanning barcode', error: err }, aomLogSrv.levels.High, aomLogSrv.severities.err);
              aomBackSrv.exit(true);
            });
        }

        $scope.barcodeSelected = function () {
          if (!$scope.readonly) {
            aomJobSrv.setField($scope.field);
            if (!window.cordova) { $scope.field.userActivated = true; }
            if ($scope.field.userActivated) {
              showBarcodeOptions($scope.field);
            } else {
              $scope.field.userActivated = true;
              scan();
            }
          }
        };
        function cancelScan() {
          aomBackSrv.exit(true);
        }
        $scope.blurBarcode = function (field) {
          $timeout(function () {
            if (ionic.Platform.isIOS() && $scope.shared !== undefined) {
              $scope.shared.keyboardUp = false;
            }
            field.manuallyEnterBarcode = false;
            aomJobSrv.calc(aomJobSrv.data.selectedField.Name);
          }, 300);
        };
      },
      templateUrl: 'App/Common/Directives/Barcode/barcode.html'
    };
  });
