﻿'use strict';
angular.module('aom.services')
  .factory('aomAPISrv', function ($http, $state, $q, $timeout, $cordovaFile, $rootScope, $ionicModal, aomNetworkSrv, aomLogSrv, aomDatabaseSrv, aomSlackSrv, aomHardwareSrv) {
    var aomApi = {};

    aomApi.data = {
      availableRootLinksUrl: 'https://s3.eu-west-2.amazonaws.com/devsites.allonmobile.com/index.json',     
      availableRootLinks: null,
      rootLink: null,
      rootLinks: null,
      config: {
        headers: {
          'Content-Type': 'application/json'
        }
      },
      fileConfig: {
        responseType: 'blob'
      },
      authData: '',
      appLoggingSecret: '4D96179D-DB6A-43BA-985C-1B074E744A71'
    };

    function pollForRootLinks() {
      $timeout(function () {
        return getRootLinks();
      }, 5000);
    }

    function getRootLinks() {
      if (aomNetworkSrv.online() && aomApi.data.rootLink !== null) {
        return $http.get(aomApi.data.rootLink)
          .then(function (response) {
            if (response === 'offline') {
              pollForRootLinks();
            } else {
              aomApi.data.rootLinks = response.data;
              aomNetworkSrv.data.rootLinksSet = true;
            }
          }, function (error) {
            pollForRootLinks();
            return handleError(error);
          });
      } else {
        pollForRootLinks();
      }
    }

    function handleError(err, errorMessage) {
      aomLogSrv.log({ message: 'API error', error: `${err} ${errorMessage ? errorMessage : ''}` }, aomLogSrv.levels.low, aomLogSrv.severities.warn);
      return $q.reject(err);
    }

    function showAvailableEnvironments() {
      var scope = $rootScope.$new();
      scope.availableRootLinks = aomApi.data.availableRootLinks;
      scope.rootLink = {
        selected: angular.copy(Enumerable.from(aomApi.data.availableRootLinks).where('$.Checked===true').toArray()[0].Code)
      };
      $ionicModal.fromTemplateUrl('App/Common/Templates/environmentSelector.html', {
        scope: scope,
        animation: aomHardwareSrv.isTomTom() ? 'none' : 'slideToLeft'
      })
        .then(function (modal) {
          scope.modal = modal;
          scope.hideModal = function () {
            aomApi.data.rootLink = scope.rootLink.selected;
            aomDatabaseSrv.insertQuery({
              RootLink: {
                link: aomApi.data.rootLink
              }
            }, true)
              .then(function () {
                $timeout(function () {
                  aomSlackSrv.init(aomApi.data.rootLink, aomApi.data.config);
                  getRootLinks();
                  scope.modal.hide()
                    .then(function () {
                      scope.modal.remove();
                    });
                });
              });
          };
          scope.modal.show();
        });
    }

    function pollForAvailableRootLinks() {
      $timeout(function () {
        return getAvailableRootLinks();
      }, 3000);
    }

    function getAvailableRootLinks() {
      if (aomNetworkSrv.online()) {
        return $http.get(aomApi.data.availableRootLinksUrl)
          .then(function (response) {
              aomApi.data.availableRootLinks = response.data;
              showAvailableEnvironments();
          }, function (error) {
            pollForAvailableRootLinks();
            return handleError(error);
          });
      } else {
        pollForAvailableRootLinks();
      }
    }
    
    aomApi.initRootLink = function () {
      aomDatabaseSrv.selectQuery({
        RootLink: {
          link: null
        }
      })
        .then(function (response) {
          if (response.rows.length === 1) {
            aomApi.data.rootLink = response.rows[0];
            aomSlackSrv.init(aomApi.data.rootLink, aomApi.data.config);
            getRootLinks();
          } else {
            if (aomApi.data.availableRootLinks != null) {
              showAvailableEnvironments();
            } else {
              getAvailableRootLinks();
            }
          }
        });
    };

    aomApi.init = function () {
      return aomApi.data.rootLinks === null ? aomApi.initRootLink() : $q.when(aomApi.data.rootLinks);
    };

    aomApi.getData = function (href, errorMessage) {
      return $http.get(href)
        .then(function (response) {
          return response.data;
        }, function (error) {
          return handleError(error, errorMessage);
        });
    };

    aomApi.getFile = function (href, errorMessage) {
      return $http.get(href, aomApi.data.fileConfig)
        .then(function (response) {
          return response;
        }, function (error) {
          return handleError(error, errorMessage);
        });
    };

    aomApi.postData = function (href, data, errorMessage) {
      return $http.post(href, data, aomApi.data.config)
        .then(function (response) {
          return response;
        }, function (error) {
          return handleError(error, errorMessage);
        });
    };

    aomApi.putData = function (href, data, errorMessage) {
      return $http.put(href, data, aomApi.data.config)
        .then(function (response) {
          return response.data;
        }, function (error) {
          return handleError(error, errorMessage);
        });
    };

    aomApi.fileUpload = function (path, fileName, mimeType, errorMessage, timestamp, type = 'job', uniqueId) {
      var def = $q.defer();
      var request = new XMLHttpRequest();

      request.onload = function () {
        if (this.status >= 200 && this.status < 300) {
          if (type === 'job') {
            var location = this.getResponseHeader('Location');
            def.resolve(location);
          } else {
            def.resolve(fileName);
          }
        } else {
          handleError('Expected 2xx');
          def.reject();
        }
      };

      request.onerror = function (error) {
        handleError(error, errorMessage);
        def.reject();
      };

      $http.get(path + '/' + fileName, { responseType: 'blob' })
        .then(function (file) {
          if (type === 'job') {
            request.open('POST', aomApi.data.rootLinks.Attachments.href);
            if (timestamp) {
              request.setRequestHeader('X-Watermark-Date', timestamp);
            }
          } else {
            var templateEngine = urltemplate.parse(aomApi.data.rootLinks.AppLogging.href);
            var objectId = `${uniqueId}_${path.split('/').slice(0, -1).pop()}${fileName}`;
            var url = templateEngine.expand({
              category: uniqueId,
              objectId: objectId
            });
            request.open('POST', url);
            request.setRequestHeader('Content-Disposition', `${aomApi.data.appLoggingSecret};filename=${uniqueId}/${objectId}`);
          }
          request.setRequestHeader('Authorization', 'Bearer ' + aomApi.data.authData.token);
          request.setRequestHeader('Content-Type', mimeType);
          request.send(file.data);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Failed to read file', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
          def.reject();
        });
      return def.promise;
    };

    return aomApi;
  });
