'use strict';
angular.module('aom.services')
  .factory('aomCleanSrv', function ($q, aomFileSystemSrv, aomAPISrv, aomResourcesSrv, aomMarkersSrv, aomDatabaseSrv, aomLogSrv, constants) {
    var clean = {},
      fieldTypes = {};

    clean.data = {};

    function createFieldTypeLookup() {
      for (var prop in constants.fieldTypes) {
        if (constants.fieldTypes.hasOwnProperty(prop)) {
          fieldTypes[constants.fieldTypes[prop]] = prop.charAt(0).toUpperCase() + prop.slice(1);
        }
      }
    }

    function fieldUpdates(job) {

      for (var i = 0; i < job.Fields.length; i++) {
        var field = job.Fields[i];

        field.visible = angular.copy(field.Options.Visibility.Default);
        field.valid = true;

        field.Parameters.valuesFromServer = angular.copy(field.Parameters.Values);
        for (var p = 0; p < field.ReferencedQueries.Parameters.length; p++) {
          var query = field.ReferencedQueries.Parameters[p];
          for (var v = 0; v < field.Parameters.Values.length; v++) {
            var value = field.Parameters.Values[v];
            if (value.Text.indexOf(query) > -1) {
              field.Parameters.Values.isSql = true;
            }
          }
        }

        field.Parameters.Values = Enumerable.from(field.Parameters.Values).where('!$.isSql').toArray();
        field.icon = aomResourcesSrv.data.resources.Job.FieldIcons[fieldTypes[field.Type]];

        if (field.Data !== null && field.Data !== '') {
          field.Data = constants.fieldTypes.time !== field.Type ? field.Data : moment(field.Data, ['h:mm']).toDate();
        }

        if (field.Attachments.length > 0) {
          field.downloadedAttachments = false;
        }

        for (var j = 0; j < field.Attachments.length; j++) {
          var attachment = field.Attachments[j];
          const attachmentFromField = job.links.attachments[attachment.LinkReference];
          attachment.filename = attachmentFromField ? attachmentFromField.filename : null;
        }

        if (field.Type === constants.fieldTypes.searchIndex) {
          field.Options.Keyboard = 2;
          field.Options.KeyboardType = 2;
          field.Parameters.Length = aomResourcesSrv.data.resources.Fields.SearchIndex.MaxLength;
        }

      }
    }

    function addWorkLocationMarker(job) {
      if (job.WorkLocation.Longitude !== null && job.WorkLocation.Latitude !== null
        && (parseFloat(job.WorkLocation.Longitude) !== 0 && parseFloat(job.WorkLocation.Latitude) !== 0)) {
        aomMarkersSrv.addMarker(job, { lat: job.WorkLocation.Latitude, lng: job.WorkLocation.Longitude, isWorkLocation: true, comment: aomResourcesSrv.data.resources.AomActionSheetOptions.WorkLocation });
        job.workLocationFromServer = true;
      } else {
        job.jsProperties.workLocationSet = false;
        job.workLocationFromServer = false;
      }
    }

    createFieldTypeLookup();

    clean.cleanBeforeJobFetch = function (job) {
      job.Paging = {
        pageHistory: [],
        selectedPage: aomResourcesSrv.data.resources.Job.FirstPageName,
        Pages: []
      };
      job.pagesOpened = [];
      job.markersOnJob = {};
      job.markerBounds = [];
    };

    clean.cleanJob = function (job) {
      job.Id = '' + job.Id;
      job.jsProperties = {
        status: job.status !== undefined ? job.status : constants.status.new,
        submitted: false,
        processing: false
      };

      job.Paging.pageHistory = [];
      job.Paging.selectedPage = aomResourcesSrv.data.resources.Job.FirstPageName;

      for (var j = 0; j < job.Attachments.length; j++) {
        var attachment = job.Attachments[j];
        const attachmentFromJob = job.links.attachments[attachment.LinkReference];
        attachment.filename = attachmentFromJob ? attachmentFromJob.filename : null;
      }

      job.confirmedReceived = false;

      var groupIndex = aomResourcesSrv.data.lastUser.groups.map(function (obj) {
        return obj.Reference;
      }).indexOf(job.ApplicationReference);

      try {
        job.Application = aomResourcesSrv.data.lastUser.groups[groupIndex].Text;
      } catch (error) {
        console.log('Error in cleanJob ' + error);
        var err = error;
        if (error && error.message && error.message === 'Cannot read property \'Text\' of undefined') {
          err = { jobNotAssignedToUser: true };
        }
        return $q.reject(err); //need to remove once Scott fixes the problem
      }

      if (job.UsersOnJob && job.UsersOnJob.length > 0) {
        job.allocatedUsers = job.UsersOnJob.filter(function (user) {
          return user.IsCurrentUser;
        }).map(function (user) {
          return user.Username;
        }).join(', ');
      }

      if (job.CompletedUsersOnJob && job.CompletedUsersOnJob.length > 0) {
        job.completedBy = job.CompletedUsersOnJob.map(function (user) {
          return user.Username + aomResourcesSrv.data.resources.Common.UserCompletedJobMessage + moment(user.DateCompleted).format(aomResourcesSrv.data.resources.Formats.LongDateFormat);
        }).join(', ');
      }

      job.pagesOpened = [];
      job.markersOnJob = {};
      job.markerBounds = [];
      job.markers = {};
      job.bounds = [];

      fieldUpdates(job);
      addWorkLocationMarker(job);


      return $q.when(job);
    };

    clean.getAttachments = function (job, attachments) {
      var attachmentPromises = [];
      for (var prop in attachments) {
        if (attachments.hasOwnProperty(prop)) {
          if (window.cordova) {
            // Use an immediately executing function to have the prop value inside the closure
            (function (prop) {
              attachmentPromises.push(clean.getAttachment(attachments[prop]));
            }(prop));
          }
        }
      }
      return $q.all(attachmentPromises);
    };

    clean.getAttachment = function (attachment, folder) {
      return aomFileSystemSrv.fileExists(attachment.filename, folder)
        .catch(function () {
          return aomAPISrv.getFile(attachment.href)
            .then(function (response) {
              return aomFileSystemSrv.write(attachment.filename, response, folder);
            });
        });
    };

    clean.cleanGroups = function (groups) {
      var selectedGroups = aomResourcesSrv.data.applicationsCopy.SelectedApplication;
      var enabledAllOptions = selectedGroups.length === 0;

      groups.map(function (group) {
        group.Checked = enabledAllOptions || !!selectedGroups.filter(item => item.Id === group.Id && item.Checked)[0];
      });

      if (enabledAllOptions) {
        angular.copy(groups, aomResourcesSrv.data.applicationsCopy.SelectedApplication);
      }

      return groups;
    };

    clean.derivedData = function (appConfig) {
      return {
        SelectedRowPadding: Enumerable.from(appConfig.RowPadding).where('$.Checked').toArray()[0],
        SelectedExtraInfo: Enumerable.from(appConfig.ExtraInfo).where('$.Checked').toArray()[0],
        SelectedTimeout: Enumerable.from(appConfig.TimeoutOptions).where('$.Checked').toArray()[0],
        SelectedSort: Enumerable.from(appConfig.SortOptions).where('$.Checked').toArray()[0],
        MapViewSelected: Enumerable.from(appConfig.MapViews).where('$.Checked').toArray()[0]
      };
    };

    clean.generateSearchData = function (job) {
      job.jsProperties.searchData = null;
      job.jsProperties.searchData = JSON.stringify(job).replace(/\"([^"]+)":/g, '');
    };

    clean.cleanAttachments = function (remoteAttachments) {
      if (!window.cordova) {
        return $q.when();
      }

      var attachments = remoteAttachments.map(function (obj) {
        return obj.filename;
      });

      var query = {
        Attachments: {
          fileName: null
        }
      };

      return aomDatabaseSrv.selectQuery(query)
        .then(function (response) {
          attachments = attachments.concat(response.rows);
          return aomFileSystemSrv.listFilenames(cordova.file.dataDirectory + constants.common.aomAttachmentsDirectory);
        })
        .then(function (entries) {
          var localAttachments = entries.filter(function (obj) {
            return !obj.isDirectory;
          }).map(function (obj) {
            return obj.name;
          });
          var toRemoveAttachments = localAttachments.filter(function (value) {
            return attachments.indexOf(value) === -1;
          });

          var promises = [];
          for (var i = 0; i < toRemoveAttachments.length; i++) {
            promises.push(aomFileSystemSrv.removeAttachment(toRemoveAttachments[i]));
          }
          return $q.all(promises);
        })
        .catch(function (err) {
          aomLogSrv.log({ message: 'Error in removing attachments', error: err }, aomLogSrv.levels.high, aomLogSrv.severities.err);
        });
    };

    return clean;
  });

