'use strict';
angular.module('aom.services')
  .factory('aomJobHelperSrv', function ($q, aomJobSrv, aomCleanSrv) {

    var jobHelperSrv = {};

    function iconCreate(cluster) {
      var colour = aomJobSrv.data.selectedJob.markers[cluster.getAllChildMarkers()[0].options.id].clusterColour;
      return new L.DivIcon({
        html: '<div style="top: -14px; left: -14px; position: relative; opacity: 0.7;"><span style="color: white;">' + cluster.getAllChildMarkers().length + '</span></div>',
        className: 'marker-cluster marker-cluster-small colour-' + colour
      });
    }

    function generateMarker(id, lat, lng, message, type, colour) {
      aomJobSrv.data.selectedJob.markers[id] = {
        lat: parseFloat(lat),
        lng: parseFloat(lng),
        layer: 'jobMarkers',
        id: id,
        message: message,
        type: type,
        icon: {
          type: 'dom',
          className: 'awesome-marker-shadow markercolour-' + colour,
          template: '<div><div class="marker"></div></div>',
          popupAnchor: [2, -7]
        },
        clusterColour: colour
      };
    }

    jobHelperSrv.createLayer = function (name, visible, type) {
      return {
        name: '' + name,
        type: type ? type : 'markercluster',
        visible: visible,
        layerOptions: {
          disableClusteringAtZoom: 18,
          singleMarkerMode: false,
          showCoverageOnHover: false,
          iconCreateFunction: iconCreate
        },
        groupOption: {
          spiderfyDistanceMultiplier: 1.5
        }
      };
    };

    jobHelperSrv.getLocations = function () {

      aomJobSrv.data.selectedJob.CustomLocations.map(function (customLocation, idx) {
        generateMarker('jobcustomlocation' + idx, customLocation.Latitude, customLocation.Longitude, customLocation.Comments, 'jobcustomlocation', 'colour01');
      });

      aomJobSrv.data.selectedJob.Attachments.filter(function (attachment) {
        return attachment.Location !== null;
      }).map(function (attachment, idx) {
        generateMarker('jobattachment' + idx, attachment.Location.Latitude, attachment.Location.Longitude, attachment.Comment, 'jobattachment', 'colour02');
      });

      for (var i = 0; i < aomJobSrv.data.selectedJob.Fields.length; i++) {
        var field = aomJobSrv.data.selectedJob.Fields[i];
        if (field.LastCustomLocation) {
          generateMarker('field' + i, field.LastCustomLocation.Latitude, field.LastCustomLocation.Longitude, field.DisplayName, 'field', 'colour03');
        }

        field.Attachments.filter(function (attachment) {
          return attachment.Location !== null;
        }).map(function (attachment, idx) {
          generateMarker('field' + i + 'attachment' + idx, attachment.Location.Latitude, attachment.Location.Longitude, attachment.Comment ? attachment.Comment : 'Attachment', 'fieldattachment', 'colour04');
        });
      }

      if (aomJobSrv.data.selectedJob.WorkLocation !== null && aomJobSrv.data.selectedJob.WorkLocation.Latitude !== null && aomJobSrv.data.selectedJob.WorkLocation.Longitude !== null) {
        generateMarker('worklocation', parseFloat(aomJobSrv.data.selectedJob.WorkLocation.Latitude), parseFloat(aomJobSrv.data.selectedJob.WorkLocation.Longitude), 'Work Location ' + aomJobSrv.data.selectedJob.WorkLocation.Address, 'worklocation', 'colour05');
      }

      for (var key in aomJobSrv.data.selectedJob.markers) {
        aomJobSrv.data.selectedJob.bounds.push([aomJobSrv.data.selectedJob.markers[key].lat, aomJobSrv.data.selectedJob.markers[key].lng]);
      }

    };

    jobHelperSrv.getAttachments = function () {

      var downloadAttachmentPromises = function (attachments) {
        return attachments.map(function (attachment) {
          return aomCleanSrv.getAttachment(aomJobSrv.data.selectedJob.links.attachments[attachment.LinkReference]);
        });
      };

      var promises = [];

      if (aomJobSrv.data.selectedJob.Attachments.length > 0) {
        promises.push(downloadAttachmentPromises(aomJobSrv.data.selectedJob.Attachments));
      }

      aomJobSrv.data.selectedJob.Fields.filter(function (field) {
        return field.Attachments.length > 0;
      }).map(function (field) {

        var fieldPromises = downloadAttachmentPromises(field.Attachments);

        promises.push(
          $q.all(fieldPromises)
            .then(function () {
              field.downloadedAttachment = true;
            })
        );
      });

      $q.all(promises);
    };

    return jobHelperSrv;
  });