'use strict';
angular.module('aom.services')
  .factory('aomSlackSrv', function ($http, $rootScope, aomResourcesSrv, aomNetworkSrv, constants, appConfig) {
    var slack = {};
    var messagePreText = '';
    var channelId = null;
    var username = '';
    var route = 'https://slack.com/api/';
    var methods = {
      postMessage: 'chat.postMessage',
      listChannels: 'channels.list'
    };
    var postConfig = null;

    function apiPost(path, data) {
      data = data ? data : {};
      data.token = appConfig.slack.token;
      return $http({
        method: 'POST',
        url: path,
        headers: { 'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8' },
        transformRequest: function (obj) {
          var str = [];
          for (var p in obj)
            str.push(encodeURIComponent(p) + '=' + encodeURIComponent(obj[p]));
          return str.join('&');
        },
        data: data
      }).catch(console.log);
    }

    function setMessageAttachments() {
      var platform = ionic.Platform.platform(),
        version = constants.common.appVersion;
      username = angular.copy(aomResourcesSrv.data.lastUser.Username);
      messagePreText = username + ' ' + platform + ' ' + version;
    }

    function getChannel(rootLink) {
      var hostname = rootLink.split('/')[2];
      //Slack channel names can be only 21 characters
      var channelName = appConfig.id.substr(0, 5) + '-' + hostname.substr(0, hostname.length - 16).split('.').join('').split('privateapi').join('');
      channelName = channelName.substr(0, 20).toLowerCase().split(' ').join('');

      apiPost(route + methods.listChannels)
        .then(function (response) {
          var filteredChannels = response.data.channels
            .filter(function (obj) {
              return obj.name === channelName;
            });

          if (filteredChannels.length === 1) {
            channelId = filteredChannels[0].id;
          } else {
            var generalChannel = response.data.channels
              .filter(function (obj) {
                return obj.name === 'general';
              });
            apiPost(route + methods.postMessage, { channel: generalChannel[0].id, text: 'Please create channel: ' + channelName });
          }
        });
    }

    // Will not curently log to slack for release
    function log(event, log) {
      if (appConfig.slack.token && !appConfig.release && channelId && aomNetworkSrv.online()) {
        if (username !== aomResourcesSrv.data.lastUser.Username) { setMessageAttachments(); }
        var data = {
          channel: channelId,
          attachments: JSON.stringify([{
            pretext: messagePreText,
            text: JSON.stringify(log)
          }])
        };
        apiPost(route + methods.postMessage, data, postConfig);
      }
    }

    slack.init = function (rootLink, config) {
      postConfig = config;
      postConfig.token = appConfig.slack.token;
      setMessageAttachments();
      getChannel(rootLink);
    };

    $rootScope.$on('slackLog', log);

    return slack;
  });