'use strict';
angular.module('aom.services')
  .factory('aomSqlMapSrv', function ($q, $timeout, aomOfflineDatabasesSrv, aomResourcesSrv, constants) {
    var sqlMapService = {},
      selectedJob,
      sqlState = null,
      sqlQueue = [];

    sqlMapService.data = {
      isSQLRunning: true
    };

    function runNextInQueue() {
      sqlState = null;
      if (sqlQueue.length > 0) {
        sqlState = { fieldName: sqlQueue[0] };
        runSql(sqlQueue.shift());
      } else {
        sqlMapService.data.isSQLRunning = false;
      }
    }

    function runSql(fieldName) {
      sqlState.timeout = $timeout(function () {
        var changedField = Enumerable.from(selectedJob.Fields).where('$.Name === "' + fieldName + '"').toArray()[0];
        changedField.isEdited = changedField.DataFromServer;
        if (changedField.SqlMap.length > 0) {
          runSqlBatches(changedField.SqlMap)
            .then(function () {
              updateAfterSqlMap();
              runNextInQueue();
            });
        } else {
          runNextInQueue();
        }
      }, 200);
    }

    function runSqlBatches(batches) {
      function next(i) {
        return runSqlBatch(batches[i])
          .then(function () {
            updateFromSql(batches[i]);
            i++;
            var nextBatch = batches[i];
            if (nextBatch) {
              return next(i);
            }
          });
      }
      return next(0);
    }

    function runSqlBatch(batch) {
      var promises = [];
      for (var i = 0; i < batch.length; i++) {
        var query = angular.copy(selectedJob.Queries[batch[i]]);

        for (var m = 0; m < query.ReferencedFields.length; m++) {
          var field = Enumerable.from(selectedJob.Fields).where('$.Name === "' + query.ReferencedFields[m] + '"').toArray()[0];
          var data = field.Data;
          data = field.Type === constants.fieldTypes.date ? data !== null ? moment(data).format(aomResourcesSrv.data.resources.Formats.SqlDateFormat) : '' : data;
          data = field.Type === constants.fieldTypes.time ? data !== null ? moment(data).format(aomResourcesSrv.data.resources.Formats.SqlTimeFormat) : ''  : data;
          query.String = query.String.split('${' + query.ReferencedFields[m] + '}').join(data);
        }

        promises.push(
          function (queryString, queryName) {
            return aomOfflineDatabasesSrv.runQuery(queryString)
              .then(function (result) {
                selectedJob.Queries[queryName].result = result;
              });
          } (query.String, batch[i])
        );
      }
      return $q.all(promises);
    }

    function updateFromSql(batch) {
      for (var s = 0; s < batch.length; s++) {
        var queryName = batch[s],
          query = selectedJob.Queries[queryName];

        for (var f = 0; f < query.UsedByFields.length; f++) {
          var field = selectedJob.Fields[selectedJob.FieldsIndex[query.UsedByFields[f]]];

          // Update Data
          var dataReferencesQuery = Enumerable.from(field.ReferencedQueries.DataFromServer).where('$ === "' + queryName + '"').toArray()[0];
          if (dataReferencesQuery) {
            var newData = angular.copy(field.DataFromServer);
            for (var i = 0; i < field.ReferencedQueries.DataFromServer.length; i++) {
              var dataFromServerQuery = field.ReferencedQueries.DataFromServer[i];
              newData = newData.split('${' + dataFromServerQuery + '}').join(selectedJob.Queries[dataFromServerQuery].result);
            }

            if (!field.isEdited) {
              field.Data = newData.indexOf('null') > -1 ? '' : newData;              
            }
          }

          // Update visibility
          var visibilityReferencesQuery = Enumerable.from(field.ReferencedQueries.Visibility).where('$ === "' + queryName + '"').toArray()[0];
          if (visibilityReferencesQuery) {
            field.visible = query.result === 'null' ? angular.copy(field.Options.Visibility.Default) : query.result === '0' ? false : true;
          }

          // Update Data
          var validationReferencesQuery = Enumerable.from(field.ReferencedQueries.Validation).where('$ === "' + queryName + '"').toArray()[0];
          if (validationReferencesQuery) {
            field.valid = query.result === 'null' || query.result === '0' ? false : true;
          }

          // Update parameters
          var parametersReferencesQuery = Enumerable.from(field.ReferencedQueries.Parameters).where('$ === "' + queryName + '"').toArray()[0];
          if (parametersReferencesQuery) {
            field.Parameters.requiresSqlUpdate = true;
          }
        }
      }
    }

    function updateAfterSqlMap() {
      for (var i = 0; i < selectedJob.Fields.length; i++) {
        var field = selectedJob.Fields[i];
        if (field.tempData !== undefined) {
          field.Data = field.tempData.indexOf('null') > -1 ? '' : angular.copy(field.tempData);
          field.tempData = undefined;
        }
        if (field.Parameters.requiresSqlUpdate) {
          var tempValues = angular.copy(field.Parameters.valuesFromServer);

          for (var q = 0; q < field.ReferencedQueries.Parameters.length; q++) {
            var queryName = field.ReferencedQueries.Parameters[q];
            var query = selectedJob.Queries[queryName];
            for (var v = 0; v < tempValues.length; v++) {
              if (tempValues[v].Text.indexOf(queryName) > -1) {
                var newParameters = [];
                if (Object.prototype.toString.call(query.result) === '[object Array]') {
                  for (var r = 0; r < query.result.length; r++) {
                    newParameters.push({
                      Checked: false,                      
                      Text: tempValues[v].Text.replace('${' + queryName + '}', query.result[r]),
                      fromSql: true
                    });
                  }
                } else {
                  newParameters.push({
                    Checked: false,                    
                    Text: tempValues[v].Text.replace('${' + queryName + '}', query.result),                                        
                    fromSql: true
                  });
                }
                Array.prototype.splice.apply(tempValues, [v, 1].concat(newParameters));
              }
            }
          }
          field.Parameters.Values = angular.copy(tempValues);
          var recordToCheck = Enumerable.from(field.Parameters.Values).where('$.Text === "' + field.Data + '"').toArray()[0];
          if (recordToCheck) {
            recordToCheck.Checked = true;
          }
        }
        field.Parameters.requiresSqlUpdate = false;
      }
    }

    sqlMapService.runIndependantSql = function (job) {
      selectedJob = job;
      if (selectedJob.SqlMap.length > 0) {
        sqlMapService.data.isSQLRunning = true;
        sqlState = { fieldName: '__job' };
        sqlState.timeout = $timeout(function () {
          runSqlBatches(selectedJob.SqlMap)
            .then(function () {
              updateAfterSqlMap();
              runNextInQueue();
            });
        }, 0);
      } else {
        sqlMapService.data.isSQLRunning = false;
      }
    };

    sqlMapService.calc = function (fieldName) {
      sqlMapService.data.isSQLRunning = true;
      if (sqlState === null) {
        sqlState = { fieldName: fieldName };
        runSql(fieldName);
      } else if (sqlState.fieldName === fieldName) {
        $timeout.cancel(sqlState.timeout);
        runSql(fieldName);
      } else {
        var existingQueuePosition = sqlQueue.indexOf(fieldName);
        if (existingQueuePosition !== -1) {
          sqlQueue.splice(existingQueuePosition, 1);
        }
        sqlQueue.push(fieldName);
      }
    };

    return sqlMapService;
  });