'use strict';
angular.module('aom.services')
  .factory('aomTextScreenSrv', function ($ionicModal, $ionicPlatform, $timeout, $q, aomComponentsSrv, aomPopupSrv, aomResourcesSrv, aomHardwareSrv) {

    var textScreenSrv = {};
    var childScope = null;

    var isFocusRequired = false;

    textScreenSrv.modal = {};

    function setupScreen() {
      var deregisterHardwareBackButton = null;

      function setupHardwareBackButton() {
        deregisterHardwareBackButton = $ionicPlatform.registerBackButtonAction(
          childScope.back, 201
        );
      }

      childScope.hideModal = function () {
        if (childScope.selectObject.Parameters && childScope.selectObject.Data.length > childScope.selectObject.Parameters.Length) {
          aomPopupSrv.alert('TextAreaMaxlength', {
            title: aomResourcesSrv.data.resources.Common.Error,
            template: aomResourcesSrv.data.resources.Job.TextAreaMaxLengthError,
            onDismiss: function () {
              $timeout(function () {
                textScreenSrv.focusField();
              }, 100);
            }
          });
        } else {
          isFocusRequired = true;
          if (window.cordova && ionic.Platform.isIOS()) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
          }
          childScope.change();
          $timeout(function () {
            childScope.modal.hide()
              .then(function () {
                childScope.modal.remove();
              });
          });
          deregisterHardwareBackButton();
        }
      };

      childScope.back = function () {
        childScope.hideModal();
      };

      setupHardwareBackButton();

      childScope.clearField = function () {
        childScope.selectObject.Data = '';
        if (ionic.Platform.isAndroid()) {
          childScope.refocus();
        }
      };

      textScreenSrv.focusField = function () {
        $timeout(function () {
          var textArea = document.getElementById('textArea');
          if (textArea) {
            textArea.focus();
            aomComponentsSrv.showKeyboard();
          }
        });
      };

      childScope.refocus = function (event) {
        event ? event.stopPropagation() : angular.noop();
        if (!isFocusRequired) {
          $timeout(function () {
            textScreenSrv.focusField();
          }, 150);
        }
      };

      childScope.blurEvent = function ($event) {
        return ionic.Platform.isAndroid() ? angular.noop() : childScope.refocus($event);
      };

    }

    textScreenSrv.show = function ($scope) {
      childScope = $scope.$new();

      if (window.cordova && ionic.Platform.isIOS()) {
        cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
      }
      $ionicModal.fromTemplateUrl('App/Common/Templates/textareaScreen.html', {
        scope: childScope,
        animation: aomHardwareSrv.isTomTom() ? 'none' : 'slideToLeft'
      })
        .then(function (modal) {
          childScope.modal = modal;
          aomPopupSrv.addModal(childScope.selectObject.Name, modal);
          textScreenSrv.modal = modal;
          isFocusRequired = false;
          setupScreen($scope);
          return $q.when();
        })
        .then(function () {
          return childScope.modal.show();
        })
        .then(function () {
          textScreenSrv.focusField();
        });
    };

    textScreenSrv.hideModal = function () {
      if (Object.keys(textScreenSrv.modal).length !== 0) {
        if (window.cordova && ionic.Platform.isIOS()) {
          cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
        }
        textScreenSrv.focusField = function () {

        };
        return textScreenSrv.modal.remove();
      } else {
        return $q.when();
      }
    };

    return textScreenSrv;
  });