angular.module('aom.services')
  .factory('aomTrackingDataFetchSrv', function ($q, $timeout, aomAPISrv, aomTrackingSrv, aomTrackingHelperSrv) {

    var trackingDataFetchSrv = {};

    trackingDataFetchSrv.data = {
    };

    function determineJobsToShow(obj) {
      return obj.WorkLocation.Latitude !== null && obj.WorkLocation.Longitude !== null && aomTrackingSrv.data.selectedApplicationReferences[obj.ApplicationReference];
    }

    function processUserJobs(user, jobs) {
      return jobs.Jobs.filter(determineJobsToShow).map(function (obj) {
        obj.layer = user.Id;
        obj.id = user.Id + 'job' + obj.Id;
        var selectedGroupUser = aomTrackingSrv.data.selectedGroup.users[user.Id];
        var colour = aomTrackingSrv.data.modeState.groupMode && selectedGroupUser ? selectedGroupUser.colour : 'colour-01';
        aomTrackingSrv.generateMarker(obj.id, obj.layer, obj.WorkLocation.Latitude, obj.WorkLocation.Longitude, aomTrackingHelperSrv.jobTemplate(obj), 0.7, colour, '');
        aomTrackingSrv.generateMarkerScope(obj.id, obj);
        return obj;
      });
    }

    function processUserHistory(user, history) {
      var tempHistory = {};
      var selectedUser = aomTrackingSrv.data.modeState.selectedUser;
      for (var i = 0; i < history.UserLocations.length; i++) {
        var element = history.UserLocations[i];
        element.Day = moment(element.Day).format('YYYYMMDD');
        aomTrackingSrv.data.layerColours.jobHistory[element.Day] = 'colour0' + (i + 1);

        element.Jobs = element.Jobs.filter(determineJobsToShow).map(function (obj) {
          obj.id = user.Id + 'job' + obj.Id;
          return obj;
        });

        if (selectedUser && user.Id === selectedUser.Id && selectedUser.history[element.Day]) {
          element.checked = false;
          element.timePosition = 1;

          if (selectedUser.history[element.Day].checked) {
            element.checked = true;
            element.timePosition = selectedUser.history[element.Day].timePosition;
          }
          angular.copy(element, aomTrackingSrv.data.modeState.selectedUser.history[element.day]);
        }
        tempHistory[element.Day] = element;
      }
      return tempHistory;
    }

    function getJobs(user) {
      return aomAPISrv.getData(user.links.jobs.href)
        .then(function (jobs) {
          return aomTrackingSrv.processIfModeStateUsersStatic(processUserJobs, user, jobs);
        })
        .then(function (usersJobs) {
          aomTrackingSrv.data.modeState.users[user.Id].jobs = usersJobs;
          aomTrackingSrv.data.modeStateUsersUpdating = false;
          return aomAPISrv.getData(user.links.history.href);
        })
        .then(function (history) {
          return aomTrackingSrv.processIfModeStateUsersStatic(processUserHistory, user, history);
        })
        .then(function (usersHistory) {
          angular.copy(usersHistory, aomTrackingSrv.data.modeState.users[user.Id].history);
          aomTrackingSrv.data.modeStateUsersUpdating = false;
          return $q.when();
        })
        .catch(function () {
          return $q.when();
        });
    }

    function getSelectedUserJobs() {
      if (aomTrackingSrv.data.modeState.selectedUser) {
        return getJobs(aomTrackingSrv.data.modeState.selectedUser);
      } else {
        return $q.when();
      }
    }

    // var gettingJobs = false;

    // function getJobsSynchronous(user) {
    //   var def = $q.defer();

    //   function getJobsInternal() {
    //     if (gettingJobs) {
    //       $timeout(function () {
    //         getJobsInternal();
    //       }, 500);
    //     } else {
    //       gettingJobs = true;
    //       getJobs(user)
    //         .then(function () {
    //           def.resolve();
    //         })
    //         .catch(function () {
    //           def.resolve();
    //         })
    //         .finally(function () {
    //           gettingJobs = false;
    //         });
    //     }
    //   }

    //   getJobsInternal();
    //   return def.promise;
    // }

    function getSelectedGroupJobs() {
      var promises = [];
      if (aomTrackingSrv.data.selectedGroup !== null) {
        var groupUsers = aomTrackingSrv.data.selectedGroup.users;
        for (var prop in groupUsers) {
          if (groupUsers.hasOwnProperty(prop) && !groupUsers[prop].deleted && aomTrackingSrv.data.modeState.users[prop]) {
            // promises.push(getJobsSynchronous(aomTrackingSrv.data.selectedGroup.users[prop]));
            promises.push(getJobs(aomTrackingSrv.data.selectedGroup.users[prop]));
          }
        }
      }
      return $q.all(promises);
    }

    trackingDataFetchSrv.getUsersJobs = function () {
      if (aomTrackingSrv.data.modeState.pollForUserJobs) {
        return aomTrackingSrv.data.modeState.groupMode ? getSelectedGroupJobs() : getSelectedUserJobs();
      } else {
        return $q.when();
      }
    };

    function showUser(user, overlays) {
      if (!overlays[user.Id]) { overlays[user.Id] = user.jobLayer; }
      overlays[user.Id].visible = aomTrackingSrv.data.modeState.activeJobs && (aomTrackingSrv.data.modeState.groupMode ? user.showJobs : true);
    }

    trackingDataFetchSrv.showUserJobs = function (user, overlays) {
      showUser(user, overlays);
      return trackingDataFetchSrv.getUsersJobs();
    };

    trackingDataFetchSrv.getUserJobs = function (user, overlays) {
      showUser(user, overlays);
      return getJobs(user);
    };

    return trackingDataFetchSrv;
  });