﻿'use strict';
angular.module('aom.directives')
  .directive('aomJobMap', function ($timeout, $ionicActionSheet, leafletMarkersHelpers, aomResourcesSrv, aomLocationSrv, leafletData, aomJobSrv, aomComponentsSrv, aomNetworkSrv, aomBackSrv, aomGoogleMapsSrv) {
    return {
      scope: {},
      controller: function ($scope) {
        $scope.data = {
          map: null,
          selectedJob: aomJobSrv.data.selectedJob,
          resources: aomResourcesSrv.data.resources
        };

        var resizeTimeout = null;

        function resizeMap() {
          if (resizeTimeout !== null) {
            $timeout.cancel(resizeTimeout);
          }
          resizeTimeout = $timeout(function () {
            $scope.data.map.invalidateSize();
            $timeout(function () {
              $scope.data.map._resetView($scope.data.map.getCenter(), $scope.data.map.getZoom(), true);
            });
          }, 1000);
        }
        
        $scope.$on('MapOptions', function () {
          showMapOptions();
        });
        
        $scope.$on('ReloadJobMap', resizeMap);
        $scope.$on('leafletDirectiveMap.jobMap.resize', resizeMap);
        
        function backClicked() {
          aomBackSrv.exit(true);
        }

        function setWorkLocationMarker(marker, latitude, longitude) {
          return aomGoogleMapsSrv.getAddress(latitude, longitude)
            .then(function (address) {
              angular.extend(marker, { address: address, locationChanged: false });
              aomJobSrv.data.selectedJob.WorkLocation.Address = address;
            });
        }

        angular.extend(
          $scope, {
            defaults: {
              maxZoom: 19,
              minZoom: 3,
              zoomAnimation: true
            },
            layers: {
              baselayers: {
                googleTerrain: {
                  top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'TERRAIN',
                  name: aomResourcesSrv.data.resources.MapOptions.Terrain,
                  layerType: 'TERRAIN',
                  type: 'google'
                },
                googleHybrid: {
                  top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'HYBRID',
                  name: aomResourcesSrv.data.resources.MapOptions.Hybrid,
                  layerType: 'HYBRID',
                  type: 'google'
                },
                googleRoadmap: {
                  top: aomResourcesSrv.data.resources.derived.MapViewSelected.Code === 'ROADMAP',
                  name: aomResourcesSrv.data.resources.MapOptions.Road,
                  layerType: 'ROADMAP',
                  type: 'google'
                }
              }
            }
          }
        );

        function showMapOptions() {
          var workLocationText = aomJobSrv.data.selectedJob.jsProperties.workLocationSet
            ? aomResourcesSrv.data.resources.AomActionSheetOptions.WorkLocation : aomResourcesSrv.data.resources.AomActionSheetOptions.WorkLocationAdd;

          var options = [
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.CurrentLocationPan, key: 'CurrentLocation' },
            { text: workLocationText, key: 'WorkLocation' },
            { text: aomResourcesSrv.data.resources.AomActionSheetOptions.CustomLocationAdd, key: 'CustomLocation' }
          ];

          if (aomJobSrv.data.selectedJob.jsProperties.workLocationSet) {
            options.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.Drive, key: 'Drive' });
          }

          $ionicActionSheet.show({
            buttons: options,
            titleText: aomResourcesSrv.data.resources.AomActionSheetOptions.Title,
            cancelText: aomResourcesSrv.data.resources.Common.Cancel,
            cancel: function () {
              return true;
            },
            buttonClicked: function (index, object) {
              var latitude = $scope.data.map.getCenter().lat;
              var longitude = $scope.data.map.getCenter().lng;
              if (object.key === 'CurrentLocation') {
                aomLocationSrv.getCurrentLocation()
                  .then(function () {
                    var location = aomLocationSrv.data.lastKnownLocation;
                    $scope.data.map.panTo([location.latitude, location.longitude]);
                  });
              } else if (object.key === 'Drive') {
                aomComponentsSrv.launchNavigator(aomJobSrv.data.selectedJob.WorkLocation.Latitude, aomJobSrv.data.selectedJob.WorkLocation.Longitude);
              } else if (object.key === 'WorkLocation') {
                if (aomJobSrv.data.selectedJob.jsProperties.workLocationSet) {
                  $scope.data.map.panTo([aomJobSrv.data.selectedJob.WorkLocation.Latitude, aomJobSrv.data.selectedJob.WorkLocation.Longitude]);
                } else {
                  aomJobSrv.data.selectedJob.WorkLocation.Latitude = latitude;
                  aomJobSrv.data.selectedJob.WorkLocation.Longitude = longitude;
                  var marker = { lat: latitude, lng: longitude, isWorkLocation: true, comment: aomResourcesSrv.data.resources.AomActionSheetOptions.WorkLocation };
                  setWorkLocationMarker(marker, latitude, longitude)
                    .finally(function () {
                      aomJobSrv.addMarker(aomJobSrv.data.selectedJob, marker);
                    });
                }
              } else if (object.key === 'CustomLocation') {
                aomJobSrv.addMarker(aomJobSrv.data.selectedJob, {
                  lat: latitude,
                  lng: longitude,
                  isWorkLocation: false,
                  comment: aomResourcesSrv.data.resources.AomActionSheetOptions.CustomLocation
                });
              }
              return true;
            }
          });
        }

        $scope.$on('leafletDirectiveMarker.jobMap.dragend', function (e, args) {
          var marker = aomJobSrv.data.selectedJob.markersOnJob[args.model.id];
          marker.lat = args.leafletObject._latlng.lat;
          marker.lng = args.leafletObject._latlng.lng;
          marker.locationChanged = true;

          if (args.model.isWorkLocation) {
            aomJobSrv.data.selectedJob.WorkLocation.Latitude = args.leafletObject._latlng.lat;
            aomJobSrv.data.selectedJob.WorkLocation.Longitude = args.leafletObject._latlng.lng;
            setWorkLocationMarker(marker, marker.lat, marker.lng);
          }
        });

        $scope.$on('$destroy', function () {
          aomJobSrv.calculateBounds();
          leafletMarkersHelpers.resetMarkerGroups();
        });


        $scope.init = function () {
          return leafletData.getMap('jobMap')
            .then(function (map) {
              $scope.data.map = map;
              L.marker([aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude], { icon: aomComponentsSrv.markerColourCurrentLocation })
                .addTo(map).bindPopup(aomResourcesSrv.data.resources.MapOptions.MarkerPopupCurrentLocation);

              var currentLocationControl = aomComponentsSrv.getLeafletCurrentLocationControl(function () {
                aomLocationSrv.getCurrentLocation()
                  .then(function () {
                    var location = aomLocationSrv.data.lastKnownLocation;
                    map.panTo([location.latitude, location.longitude]);
                  });
              });

              map.addControl(currentLocationControl);
              $scope.data.map.setView(new L.LatLng(aomLocationSrv.data.lastKnownLocation.latitude, aomLocationSrv.data.lastKnownLocation.longitude), 15);
              $scope.data.map.worldCopyJump = false;
              $timeout(function () {
                map.removeControl(map.attributionControl);
              });
              $scope.data.map.setMaxBounds([
                [-90, -180],
                [90, 180]
              ]);
              aomJobSrv.setBounds();
            });
        };

        this.$onInit = function () {
          aomBackSrv.init(backClicked);
        };

      },
      templateUrl: 'App/Components/Job/jobMap.html'
    };
  });
