'use strict';
angular.module('aom.directives')
  .directive('aomSettingControl', function ($state, $ionicPopup, aomResourcesSrv, aomStyleSrv, aomJobsSrv, constants, aomPopupSrv, aomCustomStylesSrv, aomPollSrv, aomDeviceSrv, aomLogSrv, aomStateSrv, aomBackSrv) {
    return {
      scope: {
        updateSettingsView: '&'
      },
      controller: function ($scope) {

        $scope.data = {
          resources: aomResourcesSrv.data.resources,
          lastUser: aomResourcesSrv.data.lastUser,
          sqliteVersion: aomResourcesSrv.data.sqliteVersion,
          rowClass: '',
          pageLoaded: false,
          applications: {
            Data: Enumerable.from(aomResourcesSrv.data.lastUser.groups).where('$.Checked === true').select('$.Text').toArray().join(', ').toString(),
            SelectedParameterValue: aomResourcesSrv.data.applicationsCopy.SelectedApplication,
            DisplayName: aomResourcesSrv.data.resources.Settings.Application,
            Parameters: {
              Values: aomResourcesSrv.data.lastUser.groups.sort(sortByTextProperty)
            },
            changedMethod: aomJobsSrv.changedApplicationCode
          },
          sortOptions: {
            Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.SortOptions).where('$.Checked === true').toArray()[0].Text,
            SelectedParameterValue: aomResourcesSrv.data.resources.derived.SelectedSort,
            DisplayName: aomResourcesSrv.data.resources.Settings.SortJobListBy,
            Parameters: {
              Values: aomResourcesSrv.data.resources.appConfig.SortOptions
            }
          },
          timeoutOptions: {
            Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.TimeoutOptions).where('$.Checked === true').toArray()[0].Text,
            SelectedParameterValue: aomResourcesSrv.data.resources.derived.SelectedTimeout,
            DisplayName: aomResourcesSrv.data.resources.Settings.LogoutAfter,
            Parameters: {
              Values: aomResourcesSrv.data.resources.appConfig.TimeoutOptions
            }
          },
          extraInfo: {
            Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.ExtraInfo).where('$.Checked === true').toArray()[0].Text,
            SelectedParameterValue: aomResourcesSrv.data.resources.derived.SelectedExtraInfo,
            DisplayName: aomResourcesSrv.data.resources.Settings.ExtraInfo,
            Parameters: {
              Values: aomResourcesSrv.data.resources.appConfig.ExtraInfo
            },
            changedMethod: aomJobsSrv.changedApplicationCode
          },
          rowPadding: {
            Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.RowPadding).where('$.Checked === true').toArray()[0].Text,
            SelectedParameterValue: aomResourcesSrv.data.resources.derived.SelectedRowPadding,
            DisplayName: aomResourcesSrv.data.resources.Settings.RowPadding,
            Parameters: {
              Values: aomResourcesSrv.data.resources.appConfig.RowPadding
            },
            changedMethod: setRowClass
          },
          defaultMapView: {
            Data: Enumerable.from(aomResourcesSrv.data.resources.appConfig.MapViews).where('$.Checked === true').toArray()[0].Text,
            SelectedParameterValue: aomResourcesSrv.data.resources.derived.MapViewSelected,
            DisplayName: aomResourcesSrv.data.resources.Settings.MapView,
            Parameters: {
              Values: aomResourcesSrv.data.resources.appConfig.MapViews
            }
          },
          content: {},
          isAndroid: ionic.Platform.isAndroid()
        };

        setRowClass();

        function sortByTextProperty(a, b) {
          if (a.Text < b.Text)
            return -1;
          if (a.Text > b.Text)
            return 1;
          return 0;
        }

        function setRowClass() {
          $scope.data.rowClass = aomStyleSrv.getRowClass();
          $scope.updateSettingsView();
        }

        this.$onInit = function () {
          aomBackSrv.init($scope.backClicked);
        };

        $scope.backClicked = function backClicked() {
          aomBackSrv.exit();
          aomStateSrv.setRight(constants.viewStates.empty);
        };

        $scope.reset = function () {
          $ionicPopup.confirm({
            title: aomResourcesSrv.data.resources.Settings.DataResetButton,
            template: aomResourcesSrv.data.resources.Settings.DataResetMessage,
            buttons: [{
              text: aomResourcesSrv.data.resources.Common.Cancel,
              type: 'button-stable button-outline',
            },
            {
              text: aomResourcesSrv.data.resources.Common.Okay,
              type: 'button-positive',
              onTap: () => {
                aomPollSrv.stopPolling('getJobs');
                aomDeviceSrv.clean()
                  .then(function () {
                    aomPollSrv.startPolling('getJobs');
                    aomStateSrv.setLeft(constants.viewStates.inbox);
                    aomStateSrv.setRight(constants.viewStates.empty);
                    aomStateSrv.data.selectedMenuItemTitle = aomResourcesSrv.data.resources.v5.menu.Jobs.title;
                  })
                  .catch(function (err) {
                    aomPopupSrv.alert('reset', {
                      title: aomResourcesSrv.data.resources.Common.Error,
                      template: aomResourcesSrv.data.resources.Job.Errors.DataResetFail,
                      onDismiss: function () { }
                    });
                    aomLogSrv.log({ message: 'Failed to reset data.', error: err }, aomLogSrv.levels.user, aomLogSrv.severities.err);
                  });  
              } 
            }]
          });
        };

        $scope.toggleClick = function (setting, $event) {
          if ($event !== 'change') {
            $scope.data.resources.appConfig[setting.ResourceDataProperty] = !$scope.data.resources.appConfig[setting.ResourceDataProperty];
          }
          if ($event === 'change' || $event.currentTarget.id === 'parent') {
            if (aomResourcesSrv.data.resources.appConfig[setting.ResourceDataProperty] && setting.AlertOnTrueMessage) {
              aomPopupSrv.alert('toggleShowMessageOnTrue', { title: setting.Descriptor, template: setting.AlertOnTrueMessage });
            }
            if (setting.initStyles) {
              aomCustomStylesSrv.init();
            }
          }
        };

        $scope.showOptions = function showOptions() {
          $scope.$broadcast(constants.broadcastEvents.showOfflineDatabaseOptions);
        };

        $scope.$on(constants.broadcastEvents.selectSetting, function (event, settingIndex) {
          aomBackSrv.init($scope.backClicked);
          angular.copy(aomResourcesSrv.data.resources.v5.settings.Data[settingIndex], $scope.data.content);
        });
      },
      templateUrl: 'App/Components/Settings/settingcontrol.html'
    };

  });
